@@ -151,7 +151,7 @@ class VideoFrameInner(ctypes.Structure):
             rotation=self.rotation,
             render_time_ms=self.render_time_ms,
             avsync_type=self.avsync_type,
-            metadata=ctypes.string_at(self.metadata_buffer, self.metadata_size).decode() if self.metadata_buffer else None,
+            metadata=ctypes.string_at(self.metadata_buffer, self.metadata_size) if self.metadata_buffer else None,
             shared_context=self.shared_context.decode() if self.shared_context else None,
             texture_id=self.texture_id,
             matrix=self.matrix,
@@ -718,6 +718,29 @@ class RemoteVideoTrackStatsInner(ctypes.Structure):
             stats.total_active_time,
             stats.publish_duration
         )
+    
+class ColorSpaceTypeInner(ctypes.Structure):
+    _fields_ = [
+        ("primaries_id", ctypes.c_int),
+        ("transfer_id", ctypes.c_int),
+        ("matrix_id", ctypes.c_int),
+        ("range_id", ctypes.c_int)
+    ]
+    def get(self):
+        return ColorSpaceType(
+            primaries_id=self.primaries_id,
+            transfer_id=self.transfer_id,
+            matrix_id=self.matrix_id,
+            range_id=self.range_id
+        )
+    @staticmethod
+    def create(colorspace:ColorSpaceType) -> 'ColorSpaceTypeInner':
+        return ColorSpaceTypeInner(
+            primaries_id=colorspace.primaries_id,
+            transfer_id=colorspace.transfer_id,
+            matrix_id=colorspace.matrix_id,
+            range_id=colorspace.range_id
+        )
 
 
 class ExternalVideoFrameInner(ctypes.Structure):
@@ -741,7 +764,8 @@ class ExternalVideoFrameInner(ctypes.Structure):
         ("metadata_size", ctypes.c_int),
         ("alpha_buffer", ctypes.c_void_p),
         ("fill_alpha_buffer", ctypes.c_uint8),
-        ("alpha_mode", ctypes.c_int)
+        ("alpha_mode", ctypes.c_int),
+        ("color_space", ColorSpaceTypeInner)
     ]
 
     def get(self):
@@ -765,17 +789,37 @@ class ExternalVideoFrameInner(ctypes.Structure):
             metadata_size=self.metadata_size,
             alpha_buffer=self.alpha_buffer,
             fill_alpha_buffer=self.fill_alpha_buffer,
-            alpha_mode=self.alpha_mode
+            alpha_mode=self.alpha_mode,
+            color_space=self.color_space.get()
         )
 
     @staticmethod
     def create(frame: ExternalVideoFrame) -> 'ExternalVideoFrameInner':
-        c_buffer = (ctypes.c_uint8 * len(frame.buffer)).from_buffer(frame.buffer)
-        c_buffer_ptr = ctypes.cast(c_buffer, ctypes.c_void_p)
-        c_metadata = bytearray(frame.metadata.encode('utf-8'))
-        c_metadata_ptr = (ctypes.c_uint8 * len(c_metadata)).from_buffer(c_metadata)
-        c_alpha_buffer = (ctypes.c_uint8 * len(frame.alpha_buffer)).from_buffer(frame.alpha_buffer)
-        c_alpha_buffer_ptr = ctypes.cast(c_alpha_buffer, ctypes.c_void_p)
+        if frame.buffer is not None:
+            c_buffer = (ctypes.c_uint8 * len(frame.buffer)).from_buffer(frame.buffer)
+            c_buffer_ptr = ctypes.cast(c_buffer, ctypes.c_void_p)
+        else:
+            c_buffer_ptr = ctypes.c_void_p(0)
+       
+
+        #aplha_buffer and is_fill alpha_buffer 
+        if (frame.fill_alpha_buffer >0) and (frame.alpha_buffer is not None):
+            c_alpha_buffer = (ctypes.c_uint8 * len(frame.alpha_buffer)).from_buffer(frame.alpha_buffer)
+            c_alpha_buffer_ptr = ctypes.cast(c_alpha_buffer, ctypes.c_void_p)
+        else:
+            c_alpha_buffer_ptr = ctypes.c_void_p(0)
+
+        c_metadata_size = len (frame.metadata)
+        if frame.metadata is not None and c_metadata_size > 0:
+            c_metadata_ptr = (ctypes.c_uint8 * len(frame.metadata)).from_buffer(frame.metadata)
+            c_metadata_size = len(frame.metadata)
+        else:
+            c_metadata_ptr = ctypes.c_void_p(0)
+            c_metadata_size = 0
+       
+        
+
+        
         return ExternalVideoFrameInner(
             frame.type,
             frame.format,
@@ -793,10 +837,11 @@ class ExternalVideoFrameInner(ctypes.Structure):
             frame.texture_id,
             (ctypes.c_float * 16)(*frame.matrix),
             c_metadata_ptr,
-            len(c_metadata),
+            c_metadata_size,
             c_alpha_buffer_ptr,
             frame.fill_alpha_buffer,
-            frame.alpha_mode
+            frame.alpha_mode,
+            ColorSpaceTypeInner.create(frame.color_space)
         )
 
 
@@ -1086,6 +1131,7 @@ class AgoraServiceConfigInner(ctypes.Structure):
         ('audio_scenario', ctypes.c_int),
 
         ('use_string_uid', ctypes.c_int),
+        ('domain_limit', ctypes.c_int),
     ]
 
     def get(self):
@@ -1098,7 +1144,8 @@ class AgoraServiceConfigInner(ctypes.Structure):
             area_code=self.area_code,
             channel_profile=self.channel_profile,
             audio_scenario=self.audio_scenario,
-            use_string_uid=self.use_string_uid
+            use_string_uid=self.use_string_uid,
+            domain_limit=self.domain_limit
         )
 
     @staticmethod
@@ -1112,7 +1159,8 @@ class AgoraServiceConfigInner(ctypes.Structure):
             config.area_code,
             config.channel_profile,
             config.audio_scenario,
-            config.use_string_uid
+            config.use_string_uid,
+            config.domain_limit
         )
 
 
